@@ -252,6 +252,7 @@ read_table_value(yaml_parser_t *parser, int start_line, int is_display) {
 		const char *table_file_name_check = NULL;
 		yaml_event_delete(&event);
 		query = read_table_query(parser, &table_file_name_check);
+		free(table_name);
 		table_name = lou_findTable(query);
 		free(query);
 		if (!table_name)
@@ -270,6 +271,7 @@ read_table_value(yaml_parser_t *parser, int start_line, int is_display) {
 						"Table query did not match expected table: expected '%s' but got "
 						"'%s'",
 						table_file_name_check, table_file_name + 1);
+			free(table_file_name_check);
 		}
 	}
 	table = malloc(sizeof(table_value));
@@ -302,7 +304,8 @@ read_table(yaml_event_t *start_event, yaml_parser_t *parser, const char *display
 	else if (!_lou_getTranslationTable(v->name))
 		error_at_line(EXIT_FAILURE, 0, file_name, start_event->start_mark.line + 1,
 				"Table %s not valid", v->name);
-	emph_classes = lou_getEmphClasses(v->name);  // get declared emphasis classes
+	free(emph_classes);
+	emph_classes = lou_getEmphClasses(v->name);	 // get declared emphasis classes
 	table_name = strdup((char *)v->name);
 	if (!display_table) {
 		if (v->content) {
@@ -355,6 +358,7 @@ read_flags(yaml_parser_t *parser, int *testmode) {
 			error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
 					"Flag '%s' not supported\n", event.data.scalar.value);
 		}
+		yaml_event_delete(&event);
 	}
 	if (!parse_error) yaml_parse_error(parser);
 	if (event.type != YAML_MAPPING_END_EVENT) yaml_error(YAML_MAPPING_END_EVENT, &event);
@@ -599,6 +603,7 @@ read_typeforms(yaml_parser_t *parser, int len) {
 				}
 			}
 		}
+		yaml_event_delete(&event);
 	}
 	if (!parse_error) yaml_parse_error(parser);
 
@@ -700,33 +705,6 @@ read_options(yaml_parser_t *parser, int testmode, int wordLen, int translationLe
 	yaml_event_delete(&event);
 }
 
-/* see http://stackoverflow.com/questions/5117393/utf-8-strings-length-in-linux-c */
-static int
-my_strlen_utf8_c(char *s) {
-	int i = 0, j = 0;
-	while (s[i]) {
-		if ((s[i] & 0xc0) != 0x80) j++;
-		i++;
-	}
-	return j;
-}
-
-/*
- * String parsing is also done later in check_base. At this point we
- * only need it to compute the actual string length in order to be
- * able to provide error messages when parsing typeform and position arrays.
- */
-static int
-parsed_strlen(char *s) {
-	widechar *buf;
-	int len, maxlen;
-	maxlen = my_strlen_utf8_c(s);
-	buf = malloc(sizeof(widechar) * maxlen);
-	len = _lou_extParseChars(s, buf);
-	free(buf);
-	return len;
-}
-
 static void
 read_test(yaml_parser_t *parser, char **tables, const char *display_table, int testmode) {
 	yaml_event_t event;
@@ -882,6 +860,7 @@ read_tests(
 static char **
 customTableResolver(const char *tableList, const char *base) {
 	static char *dummy_table[1];
+	static char **ret;
 	char *p = (char *)tableList;
 	while (*p != '\0') {
 		if (strncmp(p, inline_table_prefix, strlen(inline_table_prefix)) == 0)
@@ -889,10 +868,15 @@ customTableResolver(const char *tableList, const char *base) {
 		while (*p != '\0' && *p != ',') p++;
 		if (*p == ',') p++;
 	}
-	return _lou_defaultTableResolver(tableList, base);
+	if (ret) {
+		for (int i = 0; ret[i]; i++) free(ret[i]);
+		free(ret);
+	}
+	ret = _lou_defaultTableResolver(tableList, base);
+	return ret;
 }
 
-#endif  // HAVE_LIBYAML
+#endif	// HAVE_LIBYAML
 
 int
 main(int argc, char *argv[]) {
@@ -938,8 +922,8 @@ main(int argc, char *argv[]) {
 #ifndef HAVE_LIBYAML
 	fprintf(stderr, "Skipping tests for %s as libyaml was not found\n", argv[1]);
 	return EXIT_SKIPPED;
-#endif  // not HAVE_LIBYAML
-#endif  // WITHOUT_YAML
+#endif	// not HAVE_LIBYAML
+#endif	// WITHOUT_YAML
 
 #ifndef WITHOUT_YAML
 #ifdef HAVE_LIBYAML
@@ -971,6 +955,7 @@ main(int argc, char *argv[]) {
 	if (i > 0)
 		if (chdir(dir_name))
 			error(EXIT_FAILURE, EIO, "Cannot change directory to %s", dir_name);
+	free(dir_name);
 
 	// register custom table resolver
 	lou_registerTableResolver(&customTableResolver);
@@ -1105,6 +1090,6 @@ main(int argc, char *argv[]) {
 
 	return errors ? 1 : 0;
 
-#endif  // HAVE_LIBYAML
-#endif  // not WITHOUT_YAML
+#endif	// HAVE_LIBYAML
+#endif	// not WITHOUT_YAML
 }
